import com.chinaeinv.einv.order.OrderFacade;
import com.chinaeinv.einv.order.OrderFacadeAgent;
import com.chinaeinv.einv.order.dto.*;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class Main {
    // 由电子发票平台分配的appCode
    private static final String appCode = "请向瑞宏网工作人员索取";
    // 密钥库文件路径
    private static final String keyStorePath = "请向瑞宏网工作人员索取";
    // 密钥库别名
    private static final String alias = "请向瑞宏网工作人员索取";
    // 密钥库密码
    private static final String password = "请向瑞宏网工作人员索取";
    // 接口URL
    private static final String apiUrl = "https://www.chinaeinv.com:943/igs/api/orderApi.jspa";
    // 销售方纳税人识别号
    private static final String testTaxpayerCode = "请向瑞宏网工作人员索取";

    private static final DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    private static OrderFacade orderFacade = new OrderFacadeAgent(appCode, keyStorePath, alias, password, apiUrl);

    private static String orderNo = UUID.randomUUID().toString();

    public static void main(String[] args) throws Exception {
        System.out.println("订单编号：" + orderNo);

        // (异步)开具蓝字发票
        testKpAsync();

        // (异步)开具红字发票
        //testChAsync();

        // (异步)对开票失败的订单重新开票
        //testRetryKpAsync();

        // (同步)取消订单
        //testCancel();

        // (同步)根据订单编号查询发票
        testCxByOrderNo();
    }

    /**
     * (异步)开具蓝字发票
     */
    private static void testKpAsync() throws Exception {
        Order order = new Order();
        // 订单编号
        order.setOrderNo(orderNo);
        // 销货方纳税人识别号
        order.setTaxpayerCode(testTaxpayerCode);
        // 用户扫码key
        order.setScanCodeKey(UUID.randomUUID().toString());
        // 店铺
        order.setShopName("第198店");
        // 店铺编号
        order.setShopCode("198");
        // 订单时间（yyyy-MM-dd HH:mm:ss）
        order.setOrderTime(LocalDateTime.now().format(df));
        // 联系人
        order.setContact("小小明222");
        // 联系电话
        order.setContactTel("18000000000");
        // 邮箱
        order.setContactMail("test@chinaeinv.cn");
        // 配送地址
        order.setShippingAddress("火星北路1024号");
        // 购货方名称，即发票抬头
        order.setCustomerName(UUID.randomUUID().toString().replace("-",""));
        order.setCustomerCode("");
        // 是否自动开票
        order.setAutoBilling(true);
        // 开票人
        order.setDrawer("张三");
        // 收款人
        order.setPayee("李四");
        // 复核人
        order.setReviewer("王五");
        // 开票金额
        order.setTotalAmount(new BigDecimal("24000"));
        // 发票备注
        order.setRemark("软件系统维护期限为2018年1月1日至2019年12月31日。");

        // 发票明细1
        OrderItem orderItem1 = new OrderItem();
        //商品编码
        orderItem1.setCode("001");
        //商品名称
        orderItem1.setName("软件开发");
        //规格型号
        orderItem1.setSpec("无");
        //税率
        orderItem1.setTaxRate(new BigDecimal("0.06"));
        //税价合计金额
        orderItem1.setAmount(new BigDecimal("10000"));
        //商品分类编码
        orderItem1.setCatalogCode("3040203000000000000");

        // 发票明细2
        OrderItem orderItem2 = new OrderItem();
        //商品编码
        orderItem2.setCode("002");
        //商品名称
        orderItem2.setName("软件系统维护");
        //规格型号
        orderItem2.setSpec("无");
        //商品单价
        orderItem2.setPrice(new BigDecimal("8000"));
        //数量
        orderItem2.setQuantity(new BigDecimal("2"));
        //单位
        orderItem2.setUom("年");
        //税率
        orderItem2.setTaxRate(new BigDecimal("0.06"));
        //税价合计金额
        orderItem2.setAmount(new BigDecimal("16000"));
        //折扣金额
        orderItem2.setDiscountAmount(new BigDecimal("2000"));
        //商品分类编码
        orderItem2.setCatalogCode("3040203000000000000");

        // 发票明细
        order.setOrderItems(Arrays.asList(orderItem1, orderItem2));

        // 扩展参数
        order.setExtendedParams(Collections.singletonMap("testExtendedNo", "12345678"));

        // 调用接口方法
        orderFacade.kpAsync(order);
    }

    /**
     * (异步)开具红字发票
     */
    private static void testChAsync() throws Exception {
        Order order = new Order();
        // 订单编号
        order.setOrderNo("OrderNo-105");
        // 销货方纳税人识别号
        order.setTaxpayerCode(testTaxpayerCode);
        // 冲红原因
        order.setReason("开票信息有误。");

        // 调用接口方法
        orderFacade.chAsync(order);
    }

    /**
     * (异步)对开票失败的订单重新开票
     */
    private static void testRetryKpAsync() throws Exception {
        Order order = new Order();
        // 订单编号
        order.setOrderNo("OrderNo-001");
        // 销货方纳税人识别号
        order.setTaxpayerCode(testTaxpayerCode);

        // 调用接口方法
        orderFacade.retryKpAsync(order);
    }

    /**
     * (同步)取消订单
     */
    private static void testCancel() throws Exception {
        Order order = new Order();
        // 订单编号
        order.setOrderNo("OrderNo-001");
        // 销货方纳税人识别号
        order.setTaxpayerCode(testTaxpayerCode);

        // 调用接口方法
        orderFacade.cancel(order);
    }

    /**
     * (同步)根据订单编号查询发票
     */
    private static void testCxByOrderNo() throws Exception {
        OrderBase orderBase = new OrderBase(orderNo, testTaxpayerCode);

        // 调用接口方法
        orderFacade.cxByOrderNo(orderBase);
    }
}
